@@ -13,7 +13,7 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
- * $Id: cert_info.c 283 2007-06-06 09:29:45Z ludovic.rousseau $
+ * $Id: cert_info.c 407 2010-04-07 13:36:19Z ludovic.rousseau $
  */
 
 #ifndef __CERT_INFO_C_
@@ -35,7 +35,7 @@
 
 /*
  * NSS dynamic oid support.
- *  NSS is able to understand new oid tags provided by the application, 
+ *  NSS is able to understand new oid tags provided by the application,
  *  including
  *  understanding new cert extensions that NSS previously did not understand.
  *  This code adds the oids for the Kerberos Principle and the Microsoft UPN
@@ -45,17 +45,17 @@
 /* kerberois oid: 1.3.6.1.5.2.2 */
 SECOidTag CERT_KerberosPN_OID = SEC_OID_UNKNOWN;
 static const unsigned char kerberosOID[] =  { 0x2b, 0x6, 0x1, 0x5, 0x2, 0x2 };
-static const SECOidData kerberosPN_Entry = 
-       { TO_ITEM(kerberosOID), SEC_OID_UNKNOWN, 
+static const SECOidData kerberosPN_Entry =
+       { TO_ITEM(kerberosOID), SEC_OID_UNKNOWN,
        "Kerberos Priniciple", CKM_INVALID_MECHANISM, INVALID_CERT_EXTENSION };
 
 SECOidTag CERT_MicrosoftUPN_OID = SEC_OID_UNKNOWN;
 /* { 1.3.6.1.4.1.311 } */
-static const unsigned char microsoftUPNOID[] =  
-        { 0x2b, 0x6, 0x1, 0x4, 0x1, 0x82, 0x37 }; /*, xxxx  */
-static const SECOidData microsoftUPN_Entry = 
-        { TO_ITEM(microsoftUPNOID), SEC_OID_UNKNOWN, 
-        "Microsoft Universal Priniciple", CKM_INVALID_MECHANISM, 
+static const unsigned char microsoftUPNOID[] =
+        { 0x2b, 0x6, 0x1, 0x4, 0x1, 0x82, 0x37, 0x14, 0x2, 0x3 };
+static const SECOidData microsoftUPN_Entry =
+        { TO_ITEM(microsoftUPNOID), SEC_OID_UNKNOWN,
+        "Microsoft Universal Priniciple", CKM_INVALID_MECHANISM,
         INVALID_CERT_EXTENSION };
 
 /* register the oid if we haven't already */
@@ -91,7 +91,7 @@ cert_GetNameElements(CERTName *name, int wantedTag)
           results[i] = NULL;
           return results[0] ? results : NULL;
         }
-        buf = (char *)malloc(decodeItem->len + 1);
+        buf = malloc(decodeItem->len + 1);
         if ( buf ) {
           memcpy(buf, decodeItem->data, decodeItem->len);
           buf[decodeItem->len] = 0;
@@ -127,6 +127,74 @@ static char **cert_info_digest(X509 *x509, ALGORITHM_TYPE algorithm) {
   return entries;
 }
 
+static char **
+cert_info_upn (X509 *x509)
+{
+    SECItem alt_name;
+    SECStatus status;
+    PRArenaPool *arena = NULL;
+    CERTGeneralName *nameList;
+    CERTGeneralName *current;
+    SECOidTag tag;
+    static char *results[CERT_INFO_SIZE] = { NULL };
+    int result = 0;
+    SECItem decoded;
+
+    DBG("Looking for ALT_NAME");
+
+    status = CERT_FindCertExtension(x509, SEC_OID_X509_SUBJECT_ALT_NAME, &alt_name);
+    if (status != SECSuccess) {
+        DBG("Not found");
+        goto no_upn;
+    }
+
+    arena = PORT_NewArena(DER_DEFAULT_CHUNKSIZE);
+    if (!arena) {
+        DBG("Could not allocate arena");
+        goto no_upn;
+    }
+
+    nameList = current = CERT_DecodeAltNameExtension(arena, &alt_name);
+    if (!nameList) {
+        DBG("Could not decode name");
+        goto no_upn;
+    }
+
+    cert_fetchOID(&CERT_MicrosoftUPN_OID, &microsoftUPN_Entry);
+    do {
+        if (current->type == certOtherName) {
+            tag = SECOID_FindOIDTag(&current->name.OthName.oid);
+            DBG1("got other name with tag %#x", tag);
+            if (tag == CERT_MicrosoftUPN_OID) {
+				status = SEC_ASN1DecodeItem(arena, &decoded,
+					SEC_UTF8StringTemplate, &current->name.OthName.name);
+                if (status == SECSuccess) {
+                    results[result] = malloc(decoded.len + 1);
+                    memcpy(results[result], decoded.data, decoded.len);
+                    results[result][decoded.len] = '\0';
+                    DBG1("Got upn: %s", results[result]);
+                    result++;
+                } else {
+                    DBG("Could not decode upn...");
+                }
+            }
+        } else {
+            DBG("not other name...");
+        }
+        current = CERT_GetNextGeneralName(current);
+    } while (current != nameList && result < CERT_INFO_MAX_ENTRIES);
+
+no_upn:
+    if (arena) {
+        PORT_FreeArena(arena, PR_FALSE);
+    }
+
+    if (alt_name.data) {
+        SECITEM_FreeItem(&alt_name, PR_FALSE);
+    }
+
+    return results;
+}
 
 /**
 * request info on certificate
@@ -174,8 +242,7 @@ char **cert_info(X509 *x509, int type, ALGORITHM_TYPE algorithm ) {
       break;
     /* need oid tag. */
     case CERT_UPN     : /* Microsoft's Universal Principal Name */
-      cert_fetchOID(&CERT_MicrosoftUPN_OID ,& microsoftUPN_Entry);
-      return cert_GetNameElements(&x509->subject, CERT_MicrosoftUPN_OID);
+      return cert_info_upn(x509);
     case CERT_UID     : /* Certificate Unique Identifier */
       return cert_GetNameElements(&x509->subject, SEC_OID_RFC1274_UID);
       break;
@@ -184,7 +251,7 @@ char **cert_info(X509 *x509, int type, ALGORITHM_TYPE algorithm ) {
     case CERT_DIGEST  : /* Certificate Signature Digest */
       if ( !algorithm ) {
         DBG("Must specify digest algorithm");
-        return NULL;		
+        return NULL;
       }
       return cert_info_digest(x509,algorithm);
     case CERT_KEY_ALG     :
@@ -363,7 +430,7 @@ static char **cert_info_kpn(X509 *x509) {
             if ( name && name->type==GEN_OTHERNAME ) {  /* test for UPN */
                 if (OBJ_cmp(name->d.otherName->type_id, krb5PrincipalName)) continue; /* object is not a UPN */
 		else {
-		    /* NOTE: 
+		    /* NOTE:
 		    from PKINIT RFC, I deduce that stored format for kerberos
 		    Principal Name is ASN1_STRING, but not sure at 100%
 		    Any help will be granted
@@ -545,7 +612,7 @@ static char **cert_info_puk(X509 *x509) {
 	    return NULL;
 	}
 	pt=key2pem(pubk);
-	if (!pt) { 
+	if (!pt) {
 	    DBG("key2pem() failed");
 	    EVP_PKEY_free(pubk);
 	    return NULL;
@@ -562,11 +629,11 @@ static int int_append(unsigned char *pt, int n) {
 	*pt++= (n&0x00ff0000) >>16;
 	*pt++= (n&0x0000ff00) >>8;
 	*pt++= (n&0x000000ff) >>0;
-	return 4;	
+	return 4;
 }
 
 /* store an string into buffer */
-static int str_append(unsigned char *pt,char *str,int len) {
+static int str_append(unsigned char *pt, const char *str, int len) {
 	memcpy(pt,str,len);
 	return len;
 }
@@ -577,11 +644,12 @@ static int BN_append(unsigned char *pt, BIGNUM *bn) {
 	int res=0;
 	int extrabyte=0;
 	int size= 1 + BN_num_bytes(bn);
-	unsigned char *buff=malloc(size);
+	unsigned char *buff;
 	if(BN_is_zero(bn)) {
 		res= int_append(pt,0);pt+=res;
 		return res;
 	}
+	buff=malloc(size);
 	*buff=0x00;
 	res=BN_bn2bin(bn,buff+1);
 	/* TODO: handle error condition */
@@ -597,7 +665,8 @@ static int BN_append(unsigned char *pt, BIGNUM *bn) {
 */
 static char **cert_info_sshpuk(X509 *x509) {
 	char **maillist;
-	char *type,*buf;
+	const char *type;
+	char *buf;
 	unsigned char *blob,*pt,*data = NULL;
 	int data_len;
 	int res;
@@ -628,13 +697,13 @@ static char **cert_info_sshpuk(X509 *x509) {
                 	res= BN_append(pt, pubk->pkey.dsa->g); pt+=res;
                 	res= BN_append(pt, pubk->pkey.dsa->pub_key); pt+=res;
 			break;
-		case EVP_PKEY_RSA: 
+		case EVP_PKEY_RSA:
 			if (!pubk->pkey.rsa) {
 				DBG("No data for public RSA key");
 				goto sshpuk_fail;
 			}
 		        /* dump key into a byte array */
-			type="ssh-rsa"; 
+			type="ssh-rsa";
 			res= int_append(pt,strlen(type)); pt+=res;
 		        res= str_append(pt,type,strlen(type)); pt+=res;
                 	res= BN_append(pt, pubk->pkey.rsa->e); pt+=res;
@@ -660,7 +729,7 @@ static char **cert_info_sshpuk(X509 *x509) {
 	maillist= cert_info_email(x509);
 	res=0;
 	if (maillist && maillist[0]) res= strlen(maillist[0]);
-	buf=malloc(1+res+strlen(type)+data_len);
+	buf=malloc(3+res+strlen(type)+data_len);
 	if (!buf) {
 		DBG("No memory to store public key dump");
 		goto sshpuk_fail;
@@ -742,7 +811,7 @@ static char **cert_info_pem(X509 *x509) {
 */
 static char **cert_key_alg(X509 *x509) {
 	static char *entries[2] = { NULL,NULL };
-	char *alg = OBJ_nid2ln(
+	const char *alg = OBJ_nid2ln(
                     OBJ_obj2nid(x509->cert_info->key->algor->algorithm));
 	entries[0]=strdup(alg);
 	return entries;
@@ -755,10 +824,10 @@ static char **cert_info_serial_number(X509 *x509) {
 	static char *entries[2] = { NULL,NULL };
 	ASN1_INTEGER *serial = X509_get_serialNumber(x509);
 	int len;
-	char *buffer = NULL, *tmp_ptr;
+	unsigned char *buffer = NULL, *tmp_ptr;
 
 	len = i2c_ASN1_INTEGER(serial, NULL);
-	
+
 	if (len < 0) {
 		return NULL;
 	}
@@ -815,7 +884,7 @@ char **cert_info(X509 *x509, int type, const char *algorithm ) {
 	    case CERT_DIGEST  : /* Certificate Signature Digest */
 		if ( !algorithm ) {
 		    DBG("Must specify digest algorithm");
-		    return NULL;		
+		    return NULL;
 		}
 		return cert_info_digest(x509,algorithm);
 	    case CERT_KEY_ALG     : /* certificate signature algorithm */
