@@ -12,7 +12,7 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
- * $Id: pkcs11_lib.c 435 2010-06-07 14:53:17Z ludovic.rousseau $
+ * $Id: pkcs11_lib.c 453 2010-10-15 07:34:12Z ludovic.rousseau $
  */
 
 #define __PKCS11_LIB_C__
@@ -44,14 +44,19 @@ int pkcs11_pass_login(pkcs11_handle_t *h, int nullok)
 
   /* get password */
   pin =getpass("PIN for token: ");
-#ifndef DEBUG_HIDE_PASSWORD
+#ifdef DEBUG_SHOW_PASSWORD
   DBG1("PIN = [%s]", pin);
 #endif
+
+  if (NULL == pin) {
+    set_error("Error encountered while reading PIN");
+    return -1;
+  }
+
   /* for safety reasons, clean PIN string from memory asap */
 
   /* check password length */
   if (!nullok && strlen(pin) == 0) {
-    memset(pin, 0, strlen(pin));
     free(pin);
     set_error("Empty passwords not allowed");
     return -1;
@@ -62,7 +67,7 @@ int pkcs11_pass_login(pkcs11_handle_t *h, int nullok)
   memset(pin, 0, strlen(pin));
   free(pin);
   if (rv != 0) {
-    /* DBG1("pkcs11_login() failed: %s", get_error()); */
+    set_error("pkcs11_login() failed: %s", get_error());
     return -1;
   }
   return 0;
@@ -198,7 +203,7 @@ int crypto_init(cert_policy *policy) {
   }
 
   if (rv != SECSuccess) {
-    DBG1("NSS_Initialize faile: %s", SECU_Strerror(PR_GetError()));
+    DBG1("NSS_Initialize failed: %s", SECU_Strerror(PR_GetError()));
     return -1;
   }
   /* register a callback */
