@@ -12,7 +12,7 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
- * $Id: uri.c 424 2010-04-10 14:52:33Z ludovic.rousseau $
+ * $Id: uri.c 460 2010-10-19 07:40:03Z ludovic.rousseau $
  */
 
 #define __URI_C_
@@ -387,6 +387,7 @@ static int get_http(uri_t *uri, unsigned char **data, size_t *length, int rec_le
   if (sock == -1) {
     freeaddrinfo(info);
     set_error("socket() failed: %s", strerror(errno));
+    return -1;
   }
   DBG("connecting...");
   rv = connect(sock, info->ai_addr, info->ai_addrlen);
@@ -397,16 +398,17 @@ static int get_http(uri_t *uri, unsigned char **data, size_t *length, int rec_le
     return -1;
   }
   /* send http 1.0 request */
-  request = malloc(21 + strlen(uri->http->path) + strlen(uri->http->host));
+  request = malloc(32 + strlen(uri->http->path) + strlen(uri->http->host));
   if (request == NULL) {
     close(sock);
     set_error("not enough free memory available");
     return -1;
   }
   sprintf(request, "GET %s HTTP/1.0\nHost: %s\n\n\n", uri->http->path, uri->http->host);
-  rv = send(sock, request, strlen(request), 0);
+  len = strlen(request);
+  rv = send(sock, request, len, 0);
   free(request);
-  if (rv <= 0) {
+  if (rv != len) {
     close(sock);
     set_error("send() failed: %s", strerror(errno));
     return -1;
