@@ -13,7 +13,7 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
- * $Id: mapper_mgr.c 399 2010-04-07 12:49:53Z ludovic.rousseau $
+ * $Id: mapper_mgr.c 445 2010-08-14 16:19:36Z ludovic.rousseau $
  */
 
 /*
@@ -264,10 +264,17 @@ char * find_user(X509 *x509) {
 	    if(! item->module->module_data->finder) {
 	    	DBG1("Mapper '%s' has no find() function",item->module->module_name);
 	    } else {
-		set_debug_level(item->module->module_data->dbg_level);
-	        login = (*item->module->module_data->finder)(x509,item->module->module_data->context);
+			int match = 0;
+
+			set_debug_level(item->module->module_data->dbg_level);
+	        login = (*item->module->module_data->finder)(x509,item->module->module_data->context, &match);
 		set_debug_level(old_level);
-	        if (login) return login;
+	    	DBG3("Mapper '%s' found %s, matched %d", item->module->module_name,login, match);
+			if (login) {
+				if (match)
+					return login;
+				free(login);
+			}
 	    }
 	    item=item->next;
 	}
