@@ -13,7 +13,7 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
- * $Id: pam_config.c 414 2010-04-07 14:09:42Z ludovic.rousseau $
+ * $Id: pam_config.c 448 2010-09-21 09:53:51Z ludovic.rousseau $
  */
 
 #define _PAM_CONFIG_C_
@@ -125,8 +125,8 @@ static void parse_config_file(void) {
 	    set_debug_level(-2);
 	configuration.debug =
 	    scconf_get_bool(root,"debug",configuration.debug);
-	/*if (configuration.debug) set_debug_level(1);
-	else set_debug_level(0); */
+	if (configuration.debug)
+	    set_debug_level(1);
 	configuration.use_first_pass =
 	    scconf_get_bool(root,"use_first_pass",configuration.use_first_pass);
 	configuration.try_first_pass =
@@ -236,7 +236,7 @@ struct configuration_st *pk_configure( int argc, const char **argv ) {
 	int i;
 	int res;
 	/* try to find a configuration file entry */
-	for (i = 1; i < argc; i++) {
+	for (i = 0; i < argc; i++) {
 	    if (strstr(argv[i],"config_file=") ) {
 		configuration.config_file=1+strchr(argv[i],'=');
 		break;
@@ -250,7 +250,7 @@ struct configuration_st *pk_configure( int argc, const char **argv ) {
 #endif
 	/* finally parse provided arguments */
 	/* dont skip argv[0] */
-	for (i = 1; i < argc; i++) {
+	for (i = 0; i < argc; i++) {
 	   if (strcmp("nullok", argv[i]) == 0) {
 		configuration.nullok = 1;
 		continue;
