@@ -12,7 +12,7 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
- * $Id: pam_pkcs11.c 437 2010-06-08 08:01:32Z ludovic.rousseau $
+ * $Id: pam_pkcs11.c 465 2010-10-19 14:50:26Z ludovic.rousseau $
  */
 
 /* We have to make this definitions before we include the pam header files! */
@@ -405,7 +405,7 @@ PAM_EXTERN int pam_sm_authenticate(pam_handle_t *pamh, int flags, int argc, cons
     }
   } else {
       snprintf(password_prompt, sizeof(password_prompt),
-		  _("Found the %s."), _(configuration->token_type));
+		  _("%s found."), _(configuration->token_type));
       pam_prompt(pamh, PAM_TEXT_INFO, NULL, password_prompt);
   }
   rv = open_pkcs11_session(ph, slot_num);
@@ -449,7 +449,7 @@ PAM_EXTERN int pam_sm_authenticate(pam_handle_t *pamh, int flags, int argc, cons
 					"pam_get_pwd() failed: %s", pam_strerror(pamh, rv));
 			return pkcs11_pam_fail;
 		}
-#ifndef DEBUG_HIDE_PASSWORD
+#ifdef DEBUG_SHOW_PASSWORD
 		DBG1("password = [%s]", password);
 #endif
 
@@ -465,8 +465,9 @@ PAM_EXTERN int pam_sm_authenticate(pam_handle_t *pamh, int flags, int argc, cons
 	}
 	else
 	{
-		pam_prompt(pamh, PAM_TEXT_INFO, NULL,
+		snprintf(password_prompt, sizeof(password_prompt),
 			_("Enter your %s PIN on the pinpad"), _(configuration->token_type));
+		pam_prompt(pamh, PAM_TEXT_INFO, NULL, password_prompt);
 		/* use pin pad */
 		password = NULL;
 	}
@@ -693,6 +694,9 @@ PAM_EXTERN int pam_sm_authenticate(pam_handle_t *pamh, int flags, int argc, cons
            pam_strerror(pamh, rv));
   }
 
+  /* unload mapper modules */
+  unload_mappers();
+
   /* close pkcs #11 session */
   rv = close_pkcs11_session(ph);
   if (rv != 0) {
