@@ -37,6 +37,10 @@
 #include <pk11pub.h>
 #endif
 
+#ifndef HAVE_DAEMON
+int daemon(int nochdir, int noclose);
+#endif
+
 #define DEF_POLLING 1    /* 1 second timeout */
 #define DEF_EXPIRE 0    /* no expire */
 #define DEF_PKCS11_MODULE "/usr/lib/opensc-pkcs11.so"
